__all__ = ["dopen",
           "style_t",
           "dstyle",
           "Debugger",
           "printD",
           "dDEBUG",
           "dINFO",
           "dWARNING",
           "dEXCEPTION"]

from datetime import datetime
from inspect import currentframe
from os.path import abspath, dirname
from pprint import pformat
from sys import stdout
from typing import TextIO, Tuple, Optional, Dict, List, Union, Any

from console_debugger.helpful.coloring_text import style_t, StyleText
from console_debugger.helpful.date_obj import ServerError
from console_debugger.helpful.template_obj import *
from console_debugger.logic.mg_send_socket import MgSendSocket


class Debugger:
	# Все экземпляры дебагеров
	_all_instance: Dict[str, object] = {}
	# Настройки для таблицы
	_global_len_rows: List[Tuple[str, int]] = []
	_global_row_board: str = ""
	# Экземпляр сокета
	_socket_obj: Optional[MgSendSocket] = None

	def write(self, __s: str) -> Any:
		"""
		Для работы с функцией print
		print("TEXT",file=a)
		"""
		if __s not in ("\n", "", " "):
			self.__call__(__s)

	def __init__(self,
	             active: bool,
	             title_name: str,
	             *,
	             fileConfig: Optional[Dict] = None,
	             style_text: Optional[dstyle] = None,
	             consoleOutput: bool = True,
	             ):
		"""
		public set:
			+ consoleOutput = Вывод в консоль
			+ style_text = Стиль отображения

		public get:
			+ title_name = Уникальное имя дебагера
			+ fileConfig = Конфигурация для файла
			+ AllInstance = Все экземпляры дебагеров
			+ AllActiveInstance() = Все активные дебагеры
			+ AllSleepInstance() = Все приостановленный дебагиры
			+ AllUseFileName() = Все используемые имена файлов
		"""

		# title_name
		self.__title_name: str = title_name

		# active
		self.__active: bool = active
		if self.__active and self.__title_name in Debugger.AllActiveInstance():
			raise NameError("A instance of a class can only have to unique title_name")

		# fileConfig
		self.__fileConfig: Optional[Dict] = fileConfig
		if fileConfig:
			self.__addFileName_in_AllUseFileName(self.__fileConfig["file"])

		# consoleOutput
		self.consoleOutput: Optional[TextIO] = stdout if consoleOutput else None

		# style_text
		self.style_text: dstyle = dstyle(len_word=len(self.__title_name)) if not style_text else style_text

		# id
		Debugger._all_instance[title_name] = self  # Это строчка должны быть выше назначения self.__id !
		self.__id = len(Debugger.AllActiveInstance()) - 1

	def __call__(self, textOutput: Union[str, StyleText], *args, sep=' ', end='\n'):
		"""
		Здесь все логика отправки данных, как вы файл, так и на экран
		"""
		if self.__active:
			if self.__fileConfig:
				with open(**self.__fileConfig) as f:
					# Сохранять в файл не стилизованный текст
					print(
						f"|{self.__title_name}|{textOutput}|",
						*args,
						sep=sep, end=end, file=f)

			if self.consoleOutput:
				# Вывод в виде Таблицы
				if Debugger._global_len_rows:
					print(self.__designerTable(style_t(textOutput.replace("\t", ""),
					                                   **self.style_text)), end='')

				#
				elif Debugger._socket_obj:
					"""
					Если сокет закрыт, то перенаправляем вывод в стандартную консоль
					"""

					"""
					# Трассировка переемных 
					1. Отображать имя переменной, если у нее есть внешняя ссылка.
					2. Если одному и тому же значению присвоено несколько переменных, 
					вы будете получить оба этих имени переменных
					"""
					callers_local_vars = currentframe().f_back.f_back.f_locals.items()
					names_var: list = [var_name for var_name, var_val in callers_local_vars if
					                   var_val is textOutput]
					names_var_str: str = f"({', '.join(names_var)})¦" if names_var else "¦"

					"""
					Чтобы данные не копировались каждый раз при передачи в функцию, делаем ссылку на текст
					Так как разделение данных происходит по ТОЧКЕ то заменяем все точки на запятые
					"""
					res: List[str] = ["{next_steep}\n{data}{name_var}\n{textOutput}\n".format(
						next_steep=f"{'-' * (len(names_var_str) + 7)}¬",
						data=datetime.now().strftime('%H:%M:%S'),
						name_var=names_var_str,
						textOutput=f"{str(textOutput)} {' '.join(str(item) for item in args)}",
					)]

					Debugger._socket_obj.PickleDataAndSendToServer(self.__id, res)

				# Без стилей
				else:
					print(
						"|{}|\t{}".format(self.__title_name,
						                  style_t(f"{textOutput} {' '.join(str(item) for item in args)}",
						                          **self.style_text)),
						*args,
						sep=sep, end=end)

	@classmethod
	def GlobalManager(cls, *, global_status: Optional[bool] = None, typePrint: Optional[str] = "grid"):
		"""
		Глобальная настройка всех экземпляров
		:param: typePrint: Отвечает за стиль вывода данных на экран
		:param: global_active: Можно on/off все экземпляры разом
		"""

		# Логика on/off всех экземпляров
		if global_status is not None:
			if global_status:
				for k, v in cls._all_instance.items():
					v.__active = True
				return None

			if not global_status:
				for k, v in cls._all_instance.items():
					v.__active = False
				return None

		# Если нет глобально статуса или, активировались все экземпляры, то обрабатываем логику стилей
		elif global_status is None or global_status:
			# Отображать таблицу в консоли
			if typePrint == "grid":
				rowBoard: str = ""
				rowWord: str = ""
				arr: List[str] = []
				for k, v in cls._all_instance.items():
					if v.__active:
						v1 = v.style_text.copy()
						v1['agl'] = 'center'  # что бы центрировать только заголовки, а не весь текст
						text = style_t(k, **v1).present_text
						rowBoard += f"+{'-' * len(text)}"
						rowWord += f"|{text}"
						arr.append(text)
						cls._global_len_rows.append((k, len(text)))

				rowWord += "|"
				rowBoard += "+"
				row: str = f"{rowBoard}\n{rowWord}\n{rowBoard.replace('-', '=')}"
				cls._global_row_board = rowBoard
				print(row)
				return None
			else:
				cls._global_len_rows.clear()
				cls._global_row_board = ""

			# Запускам менеджер сокета
			if typePrint == "socket":

				cls._socket_obj = MgSendSocket()

				if not cls._socket_obj.ConnectToServer(cls.AllActiveInstance()):
					cls._socket_obj = None
					dirs = dirname(__file__).replace("\\", "/").split("/")[:-1]
					dirs.append("main.py tui")
					raise ServerError(
						'Вероятно сервер не запущен\n{0}\nВыполните команду:\n\npython {1}\n\n{0}\n'.format("*" * 80,
						                                                                                    "/".join(
							                                                                                    dirs)
						                                                                                    ))

				return None

			else:
				cls._socket_obj = None

	# --- Other method --- #

	def active(self):
		"""
		Активировать экземпляр
		"""
		self.__active = True

	def deactivate(self):
		"""
		Деактивировать экземпляр
		"""
		self.__active = False

	def __designerTable(self, textOutput: StyleText) -> str:
		"""
		Метод для создания таблицы
		"""
		"""
		Dependencies:
			- Debugger.GlobalLenRows:
			- self.style_text: Нужно знать размеры ячейки
			- self._title: Нужно знать заголовок для идентификации
		"""
		res_print: str = ""
		for height_item in textOutput.present_text.split('\n'):
			height_item = style_t(height_item, **self.style_text).style_text
			for item in Debugger._global_len_rows:
				if item[0] == self.__title_name:
					res_print += f"|{height_item}"
				else:
					res_print += f"|{' ' * item[1]}"
			res_print += '|\n'
		res_print += f"{Debugger._global_row_board}\n"
		return res_print

	def __addFileName_in_AllUseFileName(self, file_name: str):
		"""
		Проверяет имя файла со всеми используемыми именами файлов
		Это необходимо для защиты от случайного параллельного доступа к файлам
		"""
		absFileName = abspath(file_name).replace("\\", "/")
		if not self.AllUseFileName().get(absFileName, False):
			self.__fileConfig["file"] = absFileName
		else:
			raise FileExistsError("A single instance of a class can write to only one file")

	# --- Attribute ---#

	@staticmethod
	def AllActiveInstance() -> List[str]:
		"""
		# Имена активных дебагеров
		"""
		return [v.title_name for v in Debugger._all_instance.values()
		        if v.is_active]

	@staticmethod
	def AllSleepInstance() -> List[str]:
		"""
		# Имена остановленных дебагеров
		"""
		return [v.title_name for v in Debugger._all_instance.values()
		        if not v.is_active]

	@staticmethod
	def AllUseFileName() -> dict:
		"""
		# Используемые файлы для записи
		"""
		return {v.fileConfig["file"]: v.title_name for v in
		        Debugger._all_instance.values()
		        if v.fileConfig}

	@property
	def AllInstance(self) -> Dict[str, object]:
		"""
		Получить копию всех экземпляр дебагера
		"""
		return Debugger._all_instance.copy()

	@property
	def fileConfig(self) -> Optional[Dict]:
		"""
		Получить конфигурацию файла
		"""
		return self.__fileConfig

	@property
	def title_name(self) -> str:
		"""
		Получить уникальное имя экземпляра
		"""
		return self.__title_name

	@property
	def is_active(self) -> bool:
		"""
		Проверить активный ли экземпляр
		"""
		return self.__active

	# --- Info --- #
	def __repr__(self) -> str:
		res = {"AllActiveInstance": Debugger.AllActiveInstance(),
		       "AllUseFileName": Debugger.AllUseFileName(),
		       "AllCountSleepInstance": Debugger.AllSleepInstance()}
		res.update(self.__dict__)
		return pformat(res, indent=1, width=30, depth=2)

	def __str__(self) -> str:
		return self.__title_name


if __name__ == '__main__':
	...



import os
import random
import string
import time
import unittest
from os.path import dirname

from console_debugger.helpful.coloring_text import StyleText, cprint
from console_debugger.logic.debugger import printD, Debugger, dINFO, dWARNING, dDEBUG, dEXCEPTION, dopen, style_t

# Сгенерировать случайное слово
random_word = lambda: "".join(random.choice(string.ascii_letters) for j in range(random.randint(30, 100)))

PATH_TEST = "/".join(dirname(__file__).replace("\\", "/").split("/"))


class Test_debugger(unittest.TestCase):

	def setUp(self):

		Debugger._all_instance = {}

		self.Debug = Debugger(**dDEBUG)
		self.Info = Debugger(**dINFO)
		self.Warning = Debugger(**dWARNING)
		self.TEST = Debugger(False, "TEST")

		self.TEST_File = Debugger(True, "TEST_File", fileConfig=dopen(file="debug.log",
		                                                              mode="a",
		                                                              encoding="utf-8"))

	# Проверка внутренних переменных
	def test___addFileName_in_AllUseFileName(self):

		Debugger.GlobalManager(typePrint="grid")

		self.assertEqual(self.Debug.title_name, '[DEBUG]')
		self.assertEqual(self.Debug.AllActiveInstance(), ['[DEBUG]', '[INFO]', '[WARNING]', 'TEST_File'])
		self.assertEqual(self.Debug.AllSleepInstance(), ['TEST'])
		self.assertEqual(self.Debug.AllUseFileName(),
		                 {f'{PATH_TEST}/debug.log': 'TEST_File'})
		self.assertEqual(len(self.Debug._all_instance), 5)
		self.assertEqual(self.Debug._global_len_rows,
		                 [('[DEBUG]', 25), ('[INFO]', 25), ('[WARNING]', 31), ('TEST_File', 9)])
		self.assertEqual(self.Debug._global_row_board,
		                 "+-------------------------+-------------------------+-------------------------------+---------+")
		# self.assertEqual(self.Debug.is_SendDataToSocket, True)
		self.assertEqual(self.TEST_File.fileConfig,
		                 {'buffering': 8192,
		                  'closefd': True,
		                  'encoding': 'utf-8',
		                  'errors': None,
		                  'file': f'{PATH_TEST}/debug.log',
		                  'mode': 'a',
		                  'newline': None})

		# Нельзя добавлять дебагеры с одинаковым `title_name`
		with self.assertRaises(NameError):
			Debug2 = Debugger(True, title_name="[DEBUG]")

		# Нельзя добавлять дебагеры с одинаковым именем файла `fileConfig`
		with self.assertRaises(FileExistsError):
			Debug2 = Debugger(True, title_name="test",
			                  fileConfig=dopen(file="debug.log",
			                                   mode="w"), )

	def test__str__repr__(self):
		self.assertEqual(str(self.Debug), "[DEBUG]")

		self.assertEqual(len(repr(self.Debug)), 675)
		printD(self.Debug, '1', '2', '3')

	# @unittest.skip("grid")
	def test_GlobalManager_grid(self):
		global random_word
		Debugger.GlobalManager(typePrint="grid")

		for i in range(10):
			printD(self.Debug, random_word())
			printD(self.Info, random_word())
			printD(self.Warning, random_word())
			printD(self.TEST, random_word())
			printD(self.TEST_File, random_word())

	# @unittest.skip("None")
	def test_GlobalManager_none(self):
		global random_word
		Debugger.GlobalManager(typePrint=None)
		for i in range(10):
			printD(self.Debug, random_word())
			printD(self.Info, random_word())
			printD(self.Warning, random_word())
			printD(self.TEST, random_word())
			printD(self.TEST_File, random_word())

	# @unittest.skip("socket")
	def test_GlobalManager_socket(self):
		global random_word
		Debugger.GlobalManager(typePrint="socket")

		test_name1 = "123"
		TEST_NAME2 = "".join([str(x % 10) for x in range(50)])
		TEST_NAME3 = "123"
		TEST_NAME4 = random_word()

		for i in range(0,10):
			printD(self.Debug, test_name1)
			printD(self.Debug, '1', '2', '3')
			printD(self.Debug, {'1', '2'}, ('2',), ['3'], "4")  # Обрабатывать любой тип
			printD(self.Info, TEST_NAME2)
			printD(self.Warning, TEST_NAME3)
			printD(self.TEST, random_word())
			printD(self.TEST_File, TEST_NAME4)

		for i in range(10,20):
			printD(self.Debug, i)
			printD(self.Info, i)
			printD(self.Warning, i)
			printD(self.TEST, i)
			printD(self.TEST_File, i)

		for i in range(20,30):
			print(i, file=self.Debug)
			print(i, file=self.Info)
			print(i, file=self.Warning)
			print(i, file=self.TEST_File)

	# Проверка off/on всех дебагеров
	def test_GlobalManager(self):

		self.assertEqual(self.Debug.AllActiveInstance(), ['[DEBUG]', '[INFO]', '[WARNING]', 'TEST_File'])
		self.assertEqual(self.Debug.AllSleepInstance(), ['TEST'])

		Debugger.GlobalManager(global_status=False)

		self.assertEqual(Debugger.AllActiveInstance(), [])
		self.assertEqual(Debugger.AllSleepInstance(), ['[DEBUG]', '[INFO]', '[WARNING]', 'TEST', 'TEST_File'])
		for k, v in Debugger._all_instance.items():
			self.assertEqual(v._Debugger__active, False)

		Debugger.GlobalManager(global_status=True)

		self.assertEqual(Debugger.AllSleepInstance(), [])
		self.assertEqual(Debugger.AllActiveInstance(), ['[DEBUG]', '[INFO]', '[WARNING]', 'TEST', 'TEST_File'])
		for k, v in Debugger._all_instance.items():
			self.assertEqual(v._Debugger__active, True)

	def test_local_active_deactivate(self):
		self.assertIn(self.Debug.title_name, Debugger.AllActiveInstance())
		self.Debug.active()
		self.assertIn(self.Debug.title_name, Debugger.AllActiveInstance())
		self.Debug.deactivate()
		self.assertNotIn(self.Debug.title_name, Debugger.AllActiveInstance())
		self.Debug.active()
		self.assertIn(self.Debug.title_name, Debugger.AllActiveInstance())

	def test_templates(self):
		global random_word
		Debugger._all_instance = {}
		Debugger.AllUseFileName = {}

		print("*" * 40)
		DEBUG = Debugger(**dDEBUG)
		INFO = Debugger(**dINFO)
		WARNING = Debugger(**dWARNING)
		EXCEPTION = Debugger(**dEXCEPTION)

		Debugger.GlobalManager(typePrint=None)

		printD(DEBUG, random_word())
		printD(INFO, random_word())
		printD(WARNING, random_word())
		printD(EXCEPTION, random_word())
		print("*" * 40)

	def __del__(self):
		try:
			os.remove("debug.log")
		except FileNotFoundError:
			pass

		try:
			os.remove("debug1.log")
		except FileNotFoundError:
			pass


class Test_coloring_text(unittest.TestCase):

	def test_style_t(self):
		"""
		1. Выравнивание текста
		2. Перенос длинных строк на новую строку +
			2.1 обрезание длинных строк +
			2.2 проверка не удалять элемент из пустого массива +
		3. Удлинять короткие слова +
		4. Применять стили к тексту
		"""

		self.assertEqual(str(StyleText("123", "321")), "321")
		self.assertEqual(repr(StyleText("123", "321")), "123")

		"""
		1. Выравнивание текста
		"""

		self.assertEqual(list(style_t("123", len_word=10, agl="center").style_text),
		                 [' ', ' ', ' ', '1', '2', '3', ' ', ' ', ' ', ' '])

		self.assertEqual(list(style_t("1234", len_word=10, agl="center").style_text),
		                 [' ', ' ', ' ', '1', '2', '3', '4', ' ', ' ', ' '])

		self.assertEqual(list(style_t("1234", len_word=3, agl="center").style_text),
		                 ['1', '.', '.'])

		""" 
		 2.2 проверка не удалять элемент из пустого массива +
		"""
		test_pop = style_t("", len_word=10)
		self.assertEqual(test_pop.style_text, "")
		self.assertEqual(test_pop.present_text, "")

		list_true = [
			[
				['1', '2', '3', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
				['1', '2', '3', ' ', ' ', ' ', ' ', ' ', ' ', ' '],

				['1', '2', '\n', '3', ' '],
				['1', '2', '\n', '3', ' '],
				['1', '2', '\n', '3', ' '],
				['1', '2', '\n', '3', ' '],
				['1', '2', '\n', '3', ' '],
				['1', '2', '\n', '3', ' '],
			],

			[
				['1', '2', '3', '1', '2', '3', '\t', 'g', 'f', '\n', 'g', 'S', 'D', 'F', ' ', ' ', ' ', ' ', ' ', ' '],
				['1', '2', '3', '1', '2', '3', '\t', 'g', 'f', '\n', 'g', 'S', 'D', 'F', ' ', ' ', ' ', ' ', ' ', ' '],

				['1', '2', '\n', '3', '.'],
				['1', '2', '\n', '3', '.'],

				['1', '2', '\n', '3', '\n', '1', '2', '\n', '3', '\t', '\n', 'g', 'f', '\n', 'g', 'S', '\n', 'D', 'F'],
				['1', '2', '\n', '3', '\n', '1', '.'],

				['1', '2', '\n', '3', '\n', '1', '2', '\n', '3', '\t', '\n', 'g', 'f', '\n', 'g', 'S', '\n', 'D', 'F'],
				['1', '2', '\n', '3', '\n', '1', '.'],
			],
			[
				['1', '2', '3', 'q', 'w', 'd', 'q', 'w', 'd', 'q', '\n', 'w', 'q', 'w', 'd', 'w', 'q', 'd', 'q', 's',
				 '.'],
				['1', '2', '3', 'q', 'w', 'd', 'q', 'w', 'd', 'q', '\n', 'w', 'q', 'w', 'd', 'w', 'q', 'd', 'q', 's',
				 '.'],

				['1', '2', '\n', '3', '.'],
				['1', '2', '\n', '3', '.'],

				['1', '2', '\n', '3', 'q', '\n', 'w', 'd', '\n', 'q', 'w', '\n', 'd', 'q', '\n', 'w', 'q', '\n', 'w',
				 'd', '\n', 'w', 'q', '\n', 'd', 'q', '\n', 's', '.'],

				['1', '2', '\n', '3', 'q', '\n', 'w', '.'],

				['1', '2', '\n', '3', 'q', '\n', 'w', 'd', '\n', 'q', 'w', '\n', 'd', 'q', '\n', 'w', 'q', '\n', 'w',
				 'd', '\n', 'w', 'q', '\n', 'd', 'q', '\n', 's', '.'],

				['1', '2', '\n', '3', 'q', '\n', 'w', '.'],

			],
		]

		list_test = [
			"123",
			"123\n123\tgfgSDF",
			"123qwdqwdqwqwdwqdqsscqwergtrhtregrWASDFasD123gfgSDF",
		]

		"""
		2.1 обрезание длинных строк +
		3. Удлинять короткие слова +
		"""
		for true_data, test_set in zip(list_true, list_test):
			self.assertEqual(list(style_t(test_set, len_word=10).style_text), true_data[0])
			self.assertEqual(list(style_t(test_set, len_word=10).present_text), true_data[1])

			self.assertEqual(list(style_t(test_set, len_word=2).style_text), true_data[2])
			self.assertEqual(list(style_t(test_set, len_word=2).present_text), true_data[3])

			self.assertEqual(list(style_t(test_set, len_word=2, height=10).present_text), true_data[4])
			self.assertEqual(list(style_t(test_set, len_word=2, height=3).present_text), true_data[5])

			self.assertEqual(list(style_t(test_set, len_word=2, height=10).style_text), true_data[6])
			self.assertEqual(list(style_t(test_set, len_word=2, height=3).style_text), true_data[7])

		"""
		4. Применять стили к тексту
		"""
		self.assertEqual(style_t("123", color="red", bg_color="bg_blue", attrs=["bold"]).style_text,
		                 '\x1b[1m\x1b[44m\x1b[31m123\x1b[0m')

		cprint("123", color="red", bg_color="bg_blue", attrs=["bold"])


class TestTh_MgSendSocketData(unittest.TestCase):
	# @unittest.skip("0")
	def setUp(self):
		Debugger._all_instance = {}
		self.Debug = Debugger(**dDEBUG)
		self.Info = Debugger(**dINFO)

	# @unittest.skip("0")
	def test_send_port(self):
		global random_word
		Debugger.GlobalManager(typePrint="socket")

		for i in range(5):
			printD(self.Debug, str(i))
			printD(self.Info, str(i))

		print("end1")

		time.sleep(1)

		for i in range(5):
			printD(self.Debug, str(i))
			printD(self.Info, str(i))

		print("end2")


if __name__ == '__main__':
	unittest.main()



__all__ = ["style_t", "cprint", "StyleText"]

from collections import deque
from os import getenv
from typing import List

ATTRIBUTES = {'bold': 1,
              'dark': 2,
              'underline': 3,
              'blink': 4,
              'reverse': 5,
              'concealed': 6}

BG_COLOR = {'bg_grey': 40,
            'bg_red': 41,
            'bg_green': 42,
            'bg_yellow': 43,
            'bg_blue': 44,
            'bg_magenta': 45,
            'bg_cyan': 46,
            'bg_white': 47}

COLORS = {'grey': 30,
          'red': 31,
          'green': 32,
          'yellow': 33,
          'blue': 34,
          'magenta': 35,
          'cyan': 36,
          'white': 37}


class StyleText:
    __slots__ = ("present_text", "style_text")

    def __init__(self, present_text: str, style_text: str):
        self.present_text: str = present_text
        self.style_text: str = style_text

    def __repr__(self) -> str:
        return self.present_text

    def __str__(self) -> str:
        return self.style_text


def style_t(text: str,
            color: str = None,
            bg_color: str = None,
            attrs: List[str] = None,
            len_word: int = None,
            agl=None,
            height=2
            ):
    if len_word:
        # Выравнивание текста по центру
        if agl == 'center':

            if len(text) > len_word:
                # Если длинна слова больше разрешённой, то и центрировать не нужно, просто обрезаем строку
                text = f"{text[:len_word - 2]}.."

            else:
                # Если нужно центрировать строку меньше разрешённой длины
                text = text.center(len_word, ' ')

        else:

            # Заполняем динамический массив символами, если достигаем максимума длины то добавляем перенос строки
            arr_text: deque = deque()

            for index, sbl in enumerate(text):

                # Если слово достигла максимума по ширине и высоте, то обрезаем его
                if index + 2 > len_word * height:
                    for _ in range(-index % len_word):
                        arr_text.append(".")
                    break
                # Если слово достигло максимума по ширине, но не по высоте, то добавляем перенос строки
                if index % len_word == 0:
                    arr_text.append('\n')

                # Пользовательские переносы строк нарушают структуру, поэтому лучше их игнорировать, и
                # ставить переносы строки по правилам этого алгоритма
                if text[index] != '\n':
                    # Добавляем символы в динамический массив
                    arr_text.append(sbl)
            else:
                # Заполняем пустоту у последней строчки чтобы была одинаковая длинна столбцов
                for _ in range(-len(text) % len_word):
                    arr_text.append(" ")

            # Если будет не пустой массив
            if arr_text:
                # Убираем первый ненужный перенос строки
                arr_text.popleft()

            # Конвертируем массив в строку
            text = ''.join(arr_text)

    Style_text = text

    if getenv('ANSI_COLORS_DISABLED') is None:

        fmt_str = '\033[%dm%s'

        if color is not None:
            Style_text = fmt_str % (COLORS[color], Style_text)

        if bg_color is not None:
            Style_text = fmt_str % (BG_COLOR[bg_color], Style_text)

        if attrs is not None:
            for attr in attrs:
                Style_text = fmt_str % (ATTRIBUTES[attr], Style_text)

        if Style_text != text:
            Style_text += '\033[0m'

    return StyleText(text, Style_text)


def cprint(text, color: str = None,
           bg_color: str = None,
           attrs: List[str] = None,
           sep=' ',
           end='\n',
           file=None,
           flush=False,
           len_word: int = None,
           ):
    print(style_t(text, color, bg_color, attrs, len_word).style_text, sep=sep, end=end, file=file, flush=flush)


if __name__ == '__main__':
    ...
    # cprint('Hello, World!',
    #        color='red',
    #        bg_color="bg_blue",
    #        attrs=["concealed"],
    #        file=sys.stdout)



__all__ = ["MgGetSocket"]

import os
from pickle import UnpicklingError
from socket import socket, SOCK_STREAM, gaierror, AF_UNIX
from threading import Thread
from typing import Optional, List

from console_debugger.helpful.date_obj import DataForSocket, InitTitleNameFlag, DataFlag, EndSend, SOCKET_FILE, ViewRoot


class MgGetSocket:

	def __init__(self):

		if os.path.exists(SOCKET_FILE):
			os.remove(SOCKET_FILE)

		self.user: Optional[socket] = None
		self.address = None

		# Конфигурация сокета, конфигурации должны быть одинаковые между сервером и клиентов
		self.server_sock: socket = socket(
			family=AF_UNIX,
			type=SOCK_STREAM,
		)

		# Адрес прослушивания
		self.server_sock.bind(SOCKET_FILE)

		# Разрешает сколько количество подключений
		self.server_sock.listen(1)

	# Ждать нового подключение от клиента
	def ConnectToClient(self):
		self.UserClose()
		self.user, self.address = self.server_sock.accept()  # Ждет данные от клиентов, не проходит дальше пока нет данных
		DataForSocket.SendTrueConnect(self.user)  # Отправлять можно только байты

	@staticmethod
	def RunThread(root_: ViewRoot):
		Thread(target=MgGetSocket.CheckUpdateServer,
		       args=(root_,),
		       daemon=True, ).start()

	def UserClose(self):
		if self.user:
			self.user.close()
			self.user = None

	@staticmethod
	def CheckUpdateServer(root_: ViewRoot):
		"""
		Проверять данные из сокета и обновлять внутреннею структуру
		"""

		# Server
		try:
			root_.SeverGet = MgGetSocket()
		except (gaierror, OSError) as e:
			root_.SeverGet = None
			root_.PrintInfo(f"# {e}")
			exit(0)

		root_.SeverGet.ConnectToClient()  # Ждем подключение клиента

		#
		title_name: List[str] = []

		while root_.SeverGet:

			if root_.SeverGet.user:
				if root_.SeverGet.user.fileno() != -1:  # Если сервер не отсоединился от клиента
					try:
						flag, id_, data_l = DataForSocket.GetDataObj(root_.SeverGet.user)

						if flag == DataFlag:
							root_.SendTextInIndex(id_, data_l[0])

						elif flag == InitTitleNameFlag:
							if title_name != data_l:
								title_name = data_l
								root_.UpdateTitle(data_l)

						elif flag == EndSend:
							root_.SeverGet.ConnectToClient()

					except ConnectionAbortedError:  # Если клиенту невозможно отправить данные от отключаемся
						root_.PrintInfo("Если клиенту невозможно отправить данные")
						root_.SeverGet.ConnectToClient()

					except ConnectionResetError:  # Если клиент разорвал соединение
						root_.PrintInfo("Клиент разорвал соединение")
						root_.SeverGet.ConnectToClient()

					except (UnpicklingError, EOFError):  # Не получилось распаковать данные
						root_.PrintInfo("Ошибка распаковки")

				else:  # Если сервер отсоединился от клиента, то ждать следующего подключение
					root_.SeverGet.ConnectToClient()
			else:
				root_.SeverGet.ConnectToClient()

		root_.SeverGet.close()


if __name__ == '__main__':
	...



import random
import string
import time

from console_debugger.logic.debugger import printD, Debugger, dINFO, dDEBUG

# Сгенерировать случайное слово
random_word = lambda: "".join(random.choice(string.ascii_letters) for j in range(random.randint(30, 100)))

if __name__ == '__main__':
	Debugger._all_instance = {}
	Debug = Debugger(**dDEBUG)
	Info = Debugger(**dINFO)

	Debugger.GlobalManager(typePrint="socket")

	for i in range(5):
		printD(Debug, str(i))
		printD(Info, str(i))

	print("end1")

	time.sleep(1)

	for i in range(5):
		printD(Debug, str(i))
		printD(Info, str(i))

	print("end2")



__all__ = ["ViewGui"]

from os.path import dirname
from tkinter import Tk, Frame, Button, Text, Entry, messagebox, PhotoImage
from typing import List, Optional

from console_debugger.helpful.date_obj import ViewRoot
from console_debugger.logic.mg_get_socket import MgGetSocket


class ViewGui(ViewRoot):
	GREEN = "#487861"
	BG_COLOR = "#171b22"
	BLOCK_COLOR = "#0e1117"
	TEXT_COLOR = "#cad0d9"
	SIZE_TEXT_CONSOLE = 7

	SeverGu: Optional[MgGetSocket] = None

	def __init__(self):

		self.windowTk = Tk()
		self.windowTk.title("debugger_tk")

		img = PhotoImage("{path_}/static/icons.ico".format(
			path_="/".join(dirname(__file__).replace("\\", "/").split("/")[:-1])
		))

		# self.windowTk.iconbitmap(img)
		self.windowTk.tk.call('wm', 'iconphoto', self.windowTk._w, img)
		self.windowTk.attributes("-topmost", True)
		self.windowTk.geometry(self.__get_geometer())

		self.frameConsole: Optional[Frame] = None
		self.bt1: Optional[Button] = None
		self.Arr_textWidget: List[Text] = []
		self.__construct_widget(["1", "2"])

		self.windowTk.protocol("WM_DELETE_WINDOW", self.__del)

		super().__init__()
		MgGetSocket.RunThread(self)

		self.windowTk.mainloop()

	def PrintInfo(self, text: str):
		print(text)

	def UpdateTitle(self, l_text: List[str]):
		self.__deconstruct_widget()
		self.__construct_widget(l_text)

	def SendTextInIndex(self, index: int, data: str):
		self.Arr_textWidget[index].insert("0.1", data)

	def __construct_widget(self, names_console: Optional[List[str]]):
		"""
		Создаем и размещаем виджеты
		"""

		self.frameConsole = Frame(self.windowTk,
		                          width=100,
		                          height=200)

		self.bt1 = Button(self.windowTk,
		                  text="save geometry",
		                  bg=ViewGui.GREEN,
		                  command=lambda: self.__set_geometer()
		                  )
		self.bt1.pack(fill="x")

		self.Arr_textWidget = self._form_horizon_console(names_console, self.frameConsole)

	def __deconstruct_widget(self):
		"""
		Удаляем виджеты
		"""
		self.bt1.destroy()
		self.frameConsole.destroy()

	@staticmethod
	def display_info(message: str):
		"""
		Всплывающее окно
		"""
		messagebox.showinfo("Command", message)

	def __get_geometer(self) -> str:
		"""
		Получить размер окна сохраненный в файле
		"""
		try:
			with open(f"{dirname(__file__)}/static/config.txt", "r")as f:
				return f.read()

		except FileNotFoundError:
			return "{X}x{Y}+{W}+{H}".format(
				X=600,
				Y=600,
				W=self.windowTk.winfo_screenwidth() // 2 - 160,
				H=self.windowTk.winfo_screenheight() // 2 - 160,
			)

	def __set_geometer(self):
		"""
		Записать положение и размер окна в файл
		"""
		with open(f"{dirname(__file__)}/static/config.txt", "w")as f:
			x = self.windowTk.winfo_x()
			y = self.windowTk.winfo_y()
			w = self.windowTk.winfo_width()
			h = self.windowTk.winfo_height()
			f.write(f"{w}x{h}+{x}+{y}")

	def ExecuteCommand(self, event, index_console: int, EntryInput_obj: Entry):
		"""
		Обработчик ввода

		- clear = Отчистить консоль

		- save <name> <path> = сохранить в файл
			- `name` имя файла
			- `path` путь к папке

			save test.txt D:\

		"""

		command = EntryInput_obj.get().split()

		if command[0] == "clear":
			self.Arr_textWidget[index_console].delete(1.0, "end")
			EntryInput_obj.delete(0, "end")

		elif len(command) == 3 and command[0] == "save":
			try:
				path_ = "{}/{}".format(command[2].replace("\\", "/"), command[1])
				with open(path_, "w", encoding="utf-8")as f:
					f.write(self.Arr_textWidget[index_console].get(1.0, "end"))

				ViewGui.display_info(f"{index_console} >> {path_}")
			except (FileNotFoundError, FileExistsError, PermissionError) as e:
				ViewGui.display_info(f"{e}")

			EntryInput_obj.delete(0, "end")

	def _form_horizon_console(self, names_console: List[str],
	                          frameConsoles: Frame,
	                          ) -> List[Text]:
		"""
		Формирует горизонтально консоли и отображает её

		:type frameConsole: self.frameConsole = Frame(self.windowTk, width=100, height=200)
		:param count_console: Количество созданных консолей
		:return: список с экземплярами консолей
		"""
		ptr_arr_textWidget: List[Text] = []
		index: int = 0
		count_console: int = len(names_console)
		# нумерация строк здесь начинается с единицы, а нумерация столбцов – с нуля.

		for index_console, item in enumerate(names_console):
			Cons = Frame(frameConsoles)
			EntryInput = Entry(Cons,
			                   font=('consolas', f'{ViewGui.SIZE_TEXT_CONSOLE}'),
			                   bg=ViewGui.BLOCK_COLOR, fg=ViewGui.TEXT_COLOR,
			                   insertbackground=ViewGui.TEXT_COLOR, )
			###############
			ButtonExecute: Button = Button(Cons,
			                               text=item,
			                               bg=ViewGui.BLOCK_COLOR,
			                               fg=ViewGui.TEXT_COLOR,
			                               height=1,
			                               font=('consolas', f'{ViewGui.SIZE_TEXT_CONSOLE}'),
			                               command=lambda i=index_console, e=EntryInput: self.ExecuteCommand(None, i,
			                                                                                                 e))
			###############
			txt: Text = Text(Cons,
			                 width=80,  # Количество символов по вертикали
			                 height=20,  # Количество символов по горизонтали
			                 bg=ViewGui.BG_COLOR,
			                 fg=ViewGui.TEXT_COLOR,
			                 font=('consolas', f'{ViewGui.SIZE_TEXT_CONSOLE}'),
			                 insertbackground=ViewGui.TEXT_COLOR,
			                 )
			###############
			EntryInput.bind('<Return>', lambda v, i=index_console, e=EntryInput: self.ExecuteCommand(None, i, e))
			EntryInput.insert(0, "clear")
			###############

			ButtonExecute.pack(fill="both", )
			txt.pack(expand=True, fill="both", )
			EntryInput.pack(fill="both", )

			Cons.place(relheight=1,
			           relx=index,
			           relwidth=(1 / count_console),
			           )

			index += 1 / count_console
			ptr_arr_textWidget.append(txt)

		###############
		frameConsoles.pack(expand=True, fill="both", )
		return ptr_arr_textWidget

	def __del(self):
		self.windowTk.destroy()


if __name__ == '__main__':
	...



__all__ = ["MgSendSocket"]

from datetime import datetime
from os.path import dirname
from socket import socket, SOCK_STREAM, AF_UNIX, error
from sys import argv
from typing import List

from console_debugger.helpful.date_obj import DataForSocket, SOCKET_FILE, ServerError


class MgSendSocket:

	def __init__(self):

		# Имя файла для записи данных на случай если возникнет критическая ошибка с сервером, а очередь полная
		self.FileNameSaveIfServerError: str = "{path_f}/save_file_name{name_f}.txt".format(
			path_f=dirname(argv[0]).replace('\\', '/'),
			name_f=datetime.now().strftime('%H_%M_%S'),
		)

		# Конфигурация сокета, конфигурации должны быть одинаковые между сервером и клиентов
		self.client_sock: socket = socket(
			family=AF_UNIX,
			type=SOCK_STREAM,
		)

	def PickleDataAndSendToServer(self, id_: int, text_send: List[str]):
		"""
		Сериализовать и отправить данные на сервер
		"""
		try:
			DataForSocket.SendDataObj(self.client_sock, id_, text_send)  # Отправить данные на сервер
		except error as e:
			print(f"{e} | but data save to\n{self.FileNameSaveIfServerError}")
			self._save_output_to_file(text_send[0])

	def ConnectToServer(self, init_title_name: List[str]) -> bool:
		"""
		Соединиться с сервером
		"""

		try:
			self.client_sock.connect(SOCKET_FILE)
			data = self.client_sock.recv(1024)
			# Проверка того что мы подключились именно к нужному серверу
			if DataForSocket.CheckResponseWithServer(data):
				DataForSocket.SendInitTitleName(self.client_sock, init_title_name)
				print(data.decode("utf-8"))
				return True
			else:
				raise ServerError("Сервер отправил не верный ключ подтверждения подключения")

		except (ConnectionRefusedError, OSError, ServerError) as e:
			print(f"{e}\nServerError: Ошибка сервера")
			return False

	def _save_output_to_file(self, data_str: str):
		"""
		# Метод для сохранения данных в файл, вызывать при критических ошибках с сервером, и полной очередью
		"""
		with open(self.FileNameSaveIfServerError, 'a', encoding='utf-8') as f:
			f.write(data_str)



from os.path import dirname
from sys import path

from helpful.template_obj import add_sys_path_if_not, rel_path

add_sys_path_if_not(rel_path(-2), path)

from console_debugger.tui.urwid_terminal import ViewTui
